/*-------------------------------------------------------------------
    All files in this archive are under the GNU GPL v3 license
   Copyright 2008-2009 Purdea Andrei purdeaandrei _at_ yahoo.com
---------------------------------------------------------------------
  
  controlunit.v

  Description: This module is used to generate the control signals
  for the internal and external shift registers.
  For the internal shift register the control signals generated
  are the ``intshclk'' clock, and synchronous ``load'' signal.
  For the external shift register the control signals generated
  are ``shclk'', which is the shift clock, and ``stclk'' which
  is the store clock. The external shift registers are 74hc595
  
  1. The internal shiftregister is loaded with the data to be
    sent out serially.
  2. The bits are shifted out one by one. For safety against
  differing wire delays the shift clocks are applied later, with
    a displacement to the internal register.
  3. The values shifted into the external register are stored
    to its outputs.
  4. All this repeats again...

-------------------------------------------------------------------*/

module controlunit(clk, shclk, stclk, intshclk, load);

parameter dw = 32;
// ^^ data width, can be a maximum of 48 with the current configuration
// of the step register
input clk;
// input clock
output shclk;
// ^^ shift-clock for external serial-in parallel-out shift register
output stclk;
// ^^ store-clock for external serial-in parallel-out shift register
output intshclk;
// ^^ shift-clock for internal parallel-in serial-out shift register
output load;
// ^^ synchronous load for internal parallel-in serial-out shift register

reg intshclk;
reg load;
reg shclk;
reg stclk;


reg [7:0] step;

initial begin
    step = 0;
    intshclk = 0;
    load = 0;
    shclk = 0;
    stclk = 0;
end

always @(posedge clk) begin
    intshclk <= 0; // these will be overwritten later
    load <= 0;
    shclk <= 0;
    stclk <= 0;

    step <= step + 1;
    case(step) // these steps will load the data into the internal shiftreg
        0: load <= 1'b1; 
        1: begin intshclk <= 1'b1; load <= 1'b1; end
        2: begin intshclk <= 1'b0; load <= 1'b0; end
    endcase
    // the following steps will shift out 32 bits
    // i separated the toggleing of the internal and
    // external shift clocks to safeguard against possible
    // wire delay differences.
    if (step >= 3 && step < dw + 3) begin
        shclk <= 1'b1;
        step <= step + dw + 2;
    end
    if (step >= 3 + dw + 2 && step < dw + 3 + dw + 2) begin
        shclk <= 1'b0;
        intshclk <= 1'b1;
        step <= step + dw + 2;
    end
    if (step>= 3 + dw + 2 + dw + 2 && step < dw + 3 + dw + 2 + dw + 2) begin
        intshclk <= 1'b0;
        step <= step + 1 - dw - dw - 2 - 2;
        // ^^ if i write "step <= step - 100 + 1;" here, then
        // xilinx XST does some stupid things. It actually synthesizes
        // "step - 101" instead of "step - 99".
    end
    // and now comes the tolleing of the store clock of the
    // external shift register:
    if (step == dw + 3) stclk <= 1'b1;
    if (step == dw + 3 + 1) begin stclk <= 1'b0; step <= 0; end
end

endmodule