/*-------------------------------------------------------------------
    All files in this archive are under the GNU GPL v3 license
   Copyright 2008-2009 Purdea Andrei purdeaandrei _at_ yahoo.com
---------------------------------------------------------------------
  
  extdriver.v
  
  Description: This is the "driver" module. It brings together
  the control unit, the shift register, a clock divider that
  slows down the 50mhz clock from the spartan 3e starter kit
  board, and also the rewireing of the data bits, so they will
  appear in order from the upper level of the hierarchy.
  
  It should be possible to connect ``hexdecoder'' modules
  to the data input from a higher level without too much
  stress.
  
  the ``data'' vector contains the 7(8)segment digits in
  in least significant > most significant order (in [dw-1:0]).
  Meaning theat the most significant 8seg digit will be in 
  [7:0]. Each digit is made of A, B, C, D, E, F, G, Dot.
  
  This modules ``doubledigits'' parameter can be changed if
  more then two doubledigit displays are connected to more
  daisychained shift registers on the outside.

-------------------------------------------------------------------*/

module extdriver(clk50, sout, shclk, stclk, data);
// this parameter specifies how many double digits are
// daisychained together on the extension board:
parameter doubledigits = 2;
localparam digits = doubledigits * 2;
// this is the actual number of bits: or 'data width'
localparam dw = digits * 8;


input clk50; // the 50 mhz clock on the spartan 3e starter kit.

// the following 3 signals all go to the extension board directly
output sout;  // serial data out
output shclk; // shift clock
output stclk; // store clock
//.

input [dw - 1 : 0] data;
// ^ this signal is ordered.
// The least significant digit is on bits [7:0], and
// the most significant one is on bits [dw-1:dw-8]
// the order of the bits is (from MSB->LSB):
// segments: A B C D E F G DP


// here we divide the clock by 2 ^^ 6
// that will be a frequency of 781.25 kHz:
reg [5:0] clkdivider;
initial clkdivider <= 0;
always @(posedge clk50) clkdivider <= clkdivider + 1;
wire slowclk;
assign slowclk = clkdivider[5];


wire [dw - 1 : 0] reordered_data; // here the data will be reordered
                                  // to match how the actual segments
                                  // are connected on the board
wire intshclk; // internal shift register clock
wire load;     // synchronous internal shift register load
shiftreg #(.dw(dw)) sh0
          (.clk(intshclk),        // input
           .load(load),           // input
           .data(reordered_data), // input
           .sin(1'b0),            // input
           .sout(sout));          // Output

// the control unit generates the control signals for the internal
// and the external shift registers
controlunit    #(.dw(dw)) cu0 
                (.clk(slowclk),       // input
                 .shclk(shclk),       // Output
                 .stclk(stclk),       // Output
                 .intshclk(intshclk), // Output
                 .load(load));        // Output

// and here is the reordering of the data bits:
genvar i;
generate
    for (i=0; i<doubledigits; i=i+1) begin: doubledig
        assign reordered_data[8*i+7 : 8*i] =
                  {data[i * 2 * 8 + 8 + 5], //  C1
                   data[i * 2 * 8 + 8 + 1], //  G1
                   data[i * 2 * 8 + 8 + 4], //  D1
                   data[i * 2 * 8 + 8 + 3], //  E1
                   data[i * 2 * 8 + 0 + 0], // DP0
                   data[i * 2 * 8 + 0 + 5], //  C0
                   data[i * 2 * 8 + 0 + 4], //  D0
                   data[i * 2 * 8 + 0 + 3]};//  E0

        assign reordered_data[(digits-i)*8-1 : (digits-1-i)*8] =
                  {data[i * 2 * 8 + 0 + 2], //  F0
                   data[i * 2 * 8 + 0 + 1], //  G0
                   data[i * 2 * 8 + 0 + 7], //  A0
                   data[i * 2 * 8 + 0 + 6], //  B0
                   data[i * 2 * 8 + 8 + 2], //  F1
                   data[i * 2 * 8 + 8 + 7], //  A1
                   data[i * 2 * 8 + 8 + 6], //  B1
                   data[i * 2 * 8 + 8 + 0]};// DP1
    end
endgenerate
endmodule
